from os import path
from infi.vendata.vmware_powertools.scripts.greeter import InvalidOVF

VSERVICE_TAG = '{http://www.vmware.com/schema/ovfenv}vServiceEnvironmentSection'
VCENTER_API_TAG = '{http://www.vmware.com/schema/vservice/ExtensionVService}VCenterApi'
GUEST_API_TAG = '{http://www.vmware.com/schema/vservice/ExtensionVService}GuestApi'
VCENTER_ID_ATTRIBUTE = '{http://www.vmware.com/schema/ovfenv}vCenterId'


class OVF(object):
    OVF_ENV = '/opt/vmware/etc/vami/ovfEnv.xml'

    @classmethod
    def read_ovf_env_file(cls):
        if not path.exists(cls.OVF_ENV):
            raise InvalidOVF("OVF file {} does not exist".format(cls.OVF_ENV))
        try:
            with open(cls.OVF_ENV) as fd:
                return fd.read()
        except (OSError, IOError):
            raise InvalidOVF("OVF file {} is corrupted".format(cls.OVF_ENV))

    @classmethod
    def _element_to_dict(cls, element):
        return {child.tag.split('}')[1]: child.text for child in element.getchildren()}

    @classmethod
    def _get_root_xml(cls):
        import xml.etree.ElementTree as ET
        contents = cls.read_ovf_env_file()
        if not contents:
            raise InvalidOVF("OVF file {} is empty".format(cls.OVF_ENV))
        try:
            return ET.fromstring(contents)
        except:
            raise InvalidOVF("OVF file {} is not a valid XML".format(cls.OVF_ENV))

    @classmethod
    def _get_vcenter_element(cls):
        child = cls._get_root_xml().find(VSERVICE_TAG)
        if child is None:
            raise InvalidOVF("OVF file {} does not have the {} tag".format(cls.OVF_ENV, VSERVICE_TAG))
        return child

    @classmethod
    def _get_vcenter_api_element(cls):
        child = cls._get_vcenter_element().find(VCENTER_API_TAG)
        if child is None:
            raise InvalidOVF("OVF file {} does not have the {} tag".format(cls.OVF_ENV, VCENTER_API_TAG))
        return child

    @classmethod
    def _get_guest_api_element(cls):
        child = cls._get_vcenter_element().find(GUEST_API_TAG)
        if child is None:
            raise InvalidOVF("OVF file {} does not have the {} tag".format(cls.OVF_ENV, GUEST_API_TAG))
        return child

    @classmethod
    def get_attributes(cls):
        """:returns: a dictionary with the vCenter attributes
        The expected attributes are:
            ['URL', 'IP', 'X509Thumbprint', 'HttpPort', 'Token', 'HttpsPort', 'SelfMoRef']"""
        attributes = {}
        attributes.update(cls._element_to_dict(cls._get_vcenter_api_element()))
        attributes.update(cls._element_to_dict(cls._get_guest_api_element()))
        return attributes

    @classmethod
    def get_vm_id(cls):
        return cls._get_root_xml().get(VCENTER_ID_ATTRIBUTE)
