from logging import getLogger
from infi.storagemodel.vmware import with_host as storagemodel_with_host
from infi.storagemodel.vmware import install_all_property_collectors_on_client, fetch_all_property_collectors  # noqa
from infi.pyutils.contexts import contextmanager


MAX_RETRIES = 60
WAIT_TIME = 1

logger = getLogger(__name__)


class QueuedItem(object):
    pass


class TimeoutError(Exception):
    pass


def with_host(host):
    from infi.vendata.vmware_powertools.vcenter import vCenter
    client = vCenter.get_extension_client()
    return storagemodel_with_host(client, host)


class ContextQueue(object):
    def __init__(self, max_retries=MAX_RETRIES, wait_time_in_seconds=WAIT_TIME):
        from collections import deque
        super(ContextQueue, self).__init__()
        self._deque = deque()
        self._max_retries = max_retries
        self._wait_time = wait_time_in_seconds

    def get_in_line(self):
        item = QueuedItem()
        self._deque.appendleft(item)
        return item

    def _is_this_your_turn(self, item):
        return self._deque[-1] is item

    @contextmanager
    def wait_for_your_turn(self):
        from time import sleep
        counter = 0
        item = self.get_in_line()
        try:
            while counter < self._max_retries:
                if self._is_this_your_turn(item):
                    break
                counter += 1
                sleep(self._wait_time)
            if counter == self._max_retries:
                logger.debug("Error, queue is not empty: {!r}".format(self._deque))
                raise TimeoutError()
            yield
        finally:
            self._deque.remove(item)
