from infi.pyutils import patch
from gevent.local import local


local = local()


def set_esx_host_in_local_storage(esx_host):
    local.esx_host = esx_host
    local.get_os_hostname = lambda: esx_host.summary.config.name
    local.get_os_system = lambda: esx_host.summary.config.product.name
    local.get_os_platform = lambda: esx_host.summary.config.product.fullName


def get_datastore_by_device(device):
    from infi.vendata.vmware_powertools.vcenter.shortcuts import iter_infinidat_datastores
    for datastore, infinidat_devices in iter_infinidat_datastores(local.esx_host):
        if device in infinidat_devices:
            return datastore


def get_mount_point(device):
    datastore = get_datastore_by_device(device)
    if datastore is not None and len(datastore.host) > 0:
        return datastore.host[0].mountInfo.path


def patch_all():
    from infi.vendata.vmware_powertools.__version__ import __version__
    from infi.vendata.powertools import utils
    from infi.vendata.powertools.gui.webapp.plugin.views.host import registration
    from infi.vendata.powertools.rescan import engine as rescan_engine
    from infi.vendata.powertools.metadata_sync import engine as metadata_sync_engine
    patch.monkey_patch(utils.queries, "get_powertools_version", lambda: __version__)
    patch.monkey_patch(utils.queries, "get_os_hostname", lambda: local.get_os_hostname())
    patch.monkey_patch(utils.queries, "get_os_system", lambda: local.get_os_system())
    patch.monkey_patch(utils.queries, "get_os_platform", lambda: local.get_os_platform())
    patch.monkey_patch(registration, "refresh_storage_model", lambda: None)
    patch.monkey_patch(rescan_engine, "clear_unit_attention_from_infinidat_storage_controllers", lambda: None)
    patch.monkey_patch(metadata_sync_engine, "get_device_filesystem_name", lambda device: "VMFS")
    patch.monkey_patch(utils, "get_mount_point", get_mount_point)
