import click
from infi.vendata.vmware_powertools.__version__ import __version__


INVALID_OVF_MESSAGE = """{}.
Is the vService configured properly?
For troubleshooting tips, see:
https://support.infinidat.com/hc/articles/202319412
"""

NO_IP_ADDRESS_MESSAGE = """Network interface {} does not have an IP address.
For troubleshooting tips, see:
https://support.infinidat.com/hc/articles/202319412
"""

VCENTER_CONFIGURATION_ERROR = """vCenter configuration is invalid: {}.
For troubleshooting tips, see:
https://support.infinidat.com/hc/articles/202319412
"""

VCENTER_CONNECTIVITY_ERROR = """Connection to vCenter failed: {}.

Troubleshooting hints:
* Is the address mentioned above the actual address of the vCenter?
* Does the appliance have IP connectivity to the address mentioned above?

For troubleshooting tips, see:
https://support.infinidat.com/hc/articles/202319412
"""

PLUGIN_REGISTRATION_FAILURE_MESSAGE = """Plug-in registration has failed.
First attempt to register at {} failed with {}.
Second attempt to register at {} failed with {}.

Troubleshooting hints:
* Is the address mentioned above the actual address of the vCenter?
* Does the appliance have IP connectivity to the address mentioned above?

For troubleshooting tips, see:
https://support.infinidat.com/hc/articles/202319412
"""


class PrettyError(Exception):
    def __init__(self, message, *args, **kwargs):
        super(PrettyError, self).__init__(message.format(*args, **kwargs))


class InvalidOVF(PrettyError):
    def __init__(self, message, *args, **kwargs):
        super(InvalidOVF, self).__init__(INVALID_OVF_MESSAGE, message)

class NoIPAddress(PrettyError):
    def __init__(self, interface_name):
        super(NoIPAddress, self).__init__(NO_IP_ADDRESS_MESSAGE, interface_name)


class vCenterInvalidAddress(PrettyError):
    def __init__(self, address):
        super(vCenterInvalidAddress, self).__init__(VCENTER_CONFIGURATION_ERROR, address)


class ConnectivityError(PrettyError):
    def __init__(self, error):
        super(ConnectivityError, self).__init__(VCENTER_CONNECTIVITY_ERROR, error)


class vServiceRegistrationFailure(PrettyError):
    def __init__(self, first, second):
        super(vServiceRegistrationFailure, self).__init__(PLUGIN_REGISTRATION_FAILURE_MESSAGE,
                                                          first[0], first[1], second[0], second[1])


ERROR_LOGFILE = "/tmp/infinivmware.err.log"


def rotate_error_logfile():
    from logging.handlers import RotatingFileHandler
    handler = RotatingFileHandler(ERROR_LOGFILE, backupCount=30, delay=1)
    handler.doRollover()


@click.command()
def main():
    """Runs INFINIDAT Host PowerTools for VMware greeter screen."""
    from infi.greeter import greet
    greet("INFINIDAT Host PowerTools for VMware", __version__,
          ERROR_LOGFILE,
          "/bin/sh -c 'test -e {}'".format(ERROR_LOGFILE),
          "/bin/sh -c '/sbin/status powertools | grep running'",
          "/bin/login")


if __name__ == '__main__':
    main()
