from os.path import join, sep, exists
from infi.pyutils.contexts import contextmanager

ARGV = "--log-to-file --production-environment --populate-storagemodel --with-vasa-provider --signal-upstart server --with-vwc"
INFINIVMWARE_UPSTART_SCRIPT_PATH = join(sep, 'etc', 'init', 'powertools.conf')
LOGPAGE_UPSTART_SCRIPT_PATH = join(sep, 'etc', 'init', 'powertools-log-page.conf')
TEMPLATE = """
author "INFINIDAT, Ltd."
description "INFINIDAT Host PowerTools for VMware"
version {version}
chdir {chdir}
exec su root -c 'HTTPS_PROXY=$http_proxy {exec}'
expect stop
respawn
respawn limit 10 60
start on (local-filesystems and net-device-up IFACE!=lo)
stop on runlevel [016]
"""


@contextmanager
def restart_after():
    from infi.execute import execute, execute_assert_success
    infinivmware_existed_before = exists(INFINIVMWARE_UPSTART_SCRIPT_PATH)
    logpage_existed_before = exists(LOGPAGE_UPSTART_SCRIPT_PATH)
    with open('/etc/init/tty1.conf', 'r') as f:
        greeter_existed_before = 'infinivmware-greeter' in f.read()
    yield
    if logpage_existed_before:
        execute_assert_success(["stop", "powertools-log-page"])
        execute_assert_success(["start", "powertools-log-page"])
    else:  # in upgrade, we need to start this for those versions that did not have it
        execute(["start", "powertools-log-page"])
    if infinivmware_existed_before:
        execute_assert_success(["stop", "powertools"])
        execute_assert_success(["start", "powertools"])
    if greeter_existed_before:
        execute(["stop", "tty1"])
        execute_assert_success(["start", "tty1"])


def _install_upstart_job(filepath, console_script_name, cmdline_args):
    from .. import __version__, PROJECTROOT
    script = join(PROJECTROOT, 'bin', console_script_name)
    kwargs = {'version': __version__.__version__,
              'chdir': PROJECTROOT,
              'exec': '{} {}'.format(script, cmdline_args).strip(),
              }
    config = TEMPLATE.format(**kwargs)
    with open(filepath, 'w') as fd:
        fd.write(config)


def install():  # pragma: no cover
    from .. import PROJECTROOT
    from infi.greeter import install as install_greeter
    from infi.execute import execute_assert_success
    _install_upstart_job(INFINIVMWARE_UPSTART_SCRIPT_PATH, 'infinivmware', ARGV)
    _install_upstart_job(LOGPAGE_UPSTART_SCRIPT_PATH, 'infinivmware-log', '')
    install_greeter('tty1', join(PROJECTROOT, 'bin', 'infinivmware-greeter'))
    execute_assert_success(['initctl', 'reload-configuration'])


def signal_init_that_i_am_ready():  # pragma: no cover
    # http://upstart.ubuntu.com/cookbook/#expect-stop
    # Specifies that the job's main process will raise the SIGSTOP signal to indicate that it is ready.
    # init(8) will wait for this signal before running the job's post-start script,
    # or considering the job to be running.
    from os import getpid, kill
    from signal import SIGSTOP
    kill(getpid(), SIGSTOP)
