from .snapshotstore import SnapshotStore
from infinisdk import InfiniBox
import getpass

def ask_for_credentials(infinibox_serial):
    from infinisdk.core.exceptions import APICommandException
    while True:     # loop until successful
        try:
            address = raw_input("Please input IP address for InfiniBox serial {}: ".format(infinibox_serial)).strip()
            port = int(raw_input("Please input port for InfiniBox serial {}: ".format(infinibox_serial)).strip())
            user = raw_input("Please input user for InfiniBox serial {}: ".format(infinibox_serial)).strip()
            password = getpass.getpass("Please input password for InfiniBox serial {}: ".format(infinibox_serial)).strip()
            system = InfiniBox((address, port), auth=(user, password))
            system.get_serial()
            # connection succeeded
            print
            return (address, port, user, password)
        except ValueError:
            print "Invaid value. Please try again."
        except APICommandException:
            print "Connection failed. Please try again."
        except Exception as e:
            print "Unknown failure. Please try again.", e

def get_infinibox_details(infinibox_serial, credentials_dict):
    if infinibox_serial not in credentials_dict:
        credentials_dict[infinibox_serial] = ask_for_credentials(infinibox_serial)
    return credentials_dict[infinibox_serial]

def does_snapshot_exist(snapshot_info, credentials_dict):
    infinibox_serial = snapshot_info["infinibox_serial"]
    infinibox_snapshot_id = snapshot_info["infinibox_snapshot_id"]
    address, port, user, password = get_infinibox_details(infinibox_serial, credentials_dict)
    system = InfiniBox((address, port), auth=(user, password))
    snapshot = system.volumes.safe_get(id=infinibox_snapshot_id)
    return snapshot is not None

def handle_snapshot_group(group_uuid, credentials_dict):
    group = SnapshotStore.get_group_list(group_uuid)
    # a group is a list of 3-tuples (snapshot_store, key, snapshot_id).
    # a group represents a single datastore / snapshot
    # snapshot_id is per VM and may appear several times for different datastores
    snapshot_id_exists = dict()  # cache for snapshot existence (snapshot_id -> True/False)
    group_changed = False
    for store, key, snapshot_id in list(group):     # create a copy of "group" because we may change it
        try:
            snapshot_info = SnapshotStore.get_snapshot_info(store, key, snapshot_id)
        except:
            snapshot_id_exists[snapshot_id] = False
        else:
            infinibox_serial = snapshot_info["infinibox_serial"]
            infinibox_snapshot_id = snapshot_info["infinibox_snapshot_id"]
            if snapshot_id not in snapshot_id_exists:
                snapshot_id_exists[snapshot_id] = does_snapshot_exist(snapshot_info, credentials_dict)
                if not snapshot_id_exists[snapshot_id]:
                    msg = "Found missing snapshot: volume id {} on system serial {}"
                    print msg.format(infinibox_snapshot_id, infinibox_serial)
        if not snapshot_id_exists[snapshot_id]:
            group.remove([store, key, snapshot_id])
            SnapshotStore.del_from_store(store, key, snapshot_id)
            print "Deleted snapshot id {}".format(snapshot_id)
            group_changed = True
    # update group in SnapshotStore
    if not group_changed:
        return
    if len(group) == 0:
        SnapshotStore.del_group_list(group_uuid)
    else:
        SnapshotStore.set_group_list(group_uuid, group)

def cleanup_stale_snapshots(credentials_dict=None):
    if credentials_dict is None:
        # credentials dict maps system serials to 4-tuples (address, port, user, password)
        credentials_dict = dict()
    for group_uuid in SnapshotStore.get_all_group_ids():
        handle_snapshot_group(group_uuid, credentials_dict)
    # no need to clear the cache because datastore/folder/vm views don't use caches
    return 0
