package com.infinidat.proxyServlet;

// ========================================================================
// $Id$
// Copyright 2004-2004 Mort Bay Consulting Pty. Ltd.
// ------------------------------------------------------------------------
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// http://www.apache.org/licenses/LICENSE-2.0
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
// ========================================================================

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.InetSocketAddress;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;
import java.net.URLConnection;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.Servlet;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.handler.MessageContext;

import com.vmware.vim25.ServiceContent;
import com.vmware.vim25.VimPortType;
import com.vmware.vim25.VimService;
import com.vmware.vim25.RuntimeFaultFaultMsg;
import com.vmware.vim25.ManagedObjectReference;
import com.vmware.vim25.Extension;
import com.vmware.vim25.ExtensionServerInfo;
public class ProxyServlet implements Servlet
{

    protected HashSet<String> _DontProxyHeaders = new HashSet<String>();
    {
        _DontProxyHeaders.add("proxy-connection");
        _DontProxyHeaders.add("connection");
        _DontProxyHeaders.add("keep-alive");
        _DontProxyHeaders.add("transfer-encoding");
        _DontProxyHeaders.add("te");
        _DontProxyHeaders.add("trailer");
        _DontProxyHeaders.add("proxy-authorization");
        _DontProxyHeaders.add("proxy-authenticate");
        _DontProxyHeaders.add("upgrade");
    }

    /* (non-Javadoc)
     * @see javax.servlet.Servlet#service(javax.servlet.ServletRequest, javax.servlet.ServletResponse)
     */
    public void service(ServletRequest req, ServletResponse res) throws ServletException,
            IOException
    {
        HttpServletRequest request = (HttpServletRequest)req;
        HttpServletResponse response = (HttpServletResponse)res;
        if ("CONNECT".equalsIgnoreCase(request.getMethod())) {
            handleConnect(request, response);
        }
        else {
            String uri = request.getRequestURI();
            if (request.getQueryString() != null) {
                uri += "?" + request.getQueryString();
            }

            if (_proxyTo == null) {
                _proxyTo = getPluginUrl(req.getParameter("sessionCookie"), req.getParameter("serviceUrl"));
            }

            URL url = proxyHttpURL(request.getScheme(), request.getServerName(), request.getServerPort(), uri);


            URLConnection connection = url.openConnection();
            //connection.setAllowUserInteraction(false);

            // Set method
            HttpURLConnection http = null;
            if (connection instanceof HttpURLConnection) {
                http = (HttpURLConnection)connection;
                http.setRequestMethod(request.getMethod());
                http.setInstanceFollowRedirects(true);
            }

            // check connection header
            String connectionHdr = request.getHeader("Connection");
            if (connectionHdr != null) {
                connectionHdr = connectionHdr.toLowerCase();
                if (connectionHdr.equals("keep-alive")||
                    connectionHdr.equals("close"))
                    connectionHdr = null;
            }

            // copy headers
            boolean xForwardedFor = false;
            boolean hasContent = false;
            Enumeration enm = request.getHeaderNames();
            while (enm.hasMoreElements()) {
                // TODO could be better than this!
                String hdr = (String)enm.nextElement();
                String lhdr = hdr.toLowerCase();

                if (_DontProxyHeaders.contains(lhdr))
                    continue;
                if (connectionHdr != null && connectionHdr.indexOf(lhdr)>=0)
                    continue;

                if ("content-type".equals(lhdr) || "content-length".equals(lhdr))
                    hasContent = true;

                Enumeration vals = request.getHeaders(hdr);
                while (vals.hasMoreElements()) {
                    String val = (String)vals.nextElement();
                    if (val != null) {
                        connection.addRequestProperty(hdr, val);
                        xForwardedFor |= "X-Forwarded-For".equalsIgnoreCase(hdr);
                    }
                }
            }

            // Proxy headers
            connection.setRequestProperty("Via", "INFINIDAT Proxy Plugin");
            if (!xForwardedFor) {
                connection.addRequestProperty("X-Forwarded-For", request.getRemoteAddr());
                connection.addRequestProperty("X-Forwarded-Proto", request.getScheme());
                connection.addRequestProperty("X-Forwarded-Host", request.getServerName());
                connection.addRequestProperty("X-Forwarded-Server", request.getLocalName());
            }

            // a little bit of cache control
            String cache_control = request.getHeader("Cache-Control");
            if (cache_control != null &&
                (cache_control.indexOf("no-cache") >= 0 ||
                 cache_control.indexOf("no-store") >= 0))
                connection.setUseCaches(false);

            // customize Connection

            try {
                connection.setDoInput(true);

                // do input thang!
                InputStream in = request.getInputStream();
                if (hasContent) {
                    connection.setDoOutput(true);
                    copy(in, connection.getOutputStream());
                }

                // Connect
                connection.connect();
            }
            catch (Exception e) {
            }

            InputStream proxy_in = null;

            // handler status codes etc.
            int code = 500;
            if (http != null) {
                proxy_in = http.getErrorStream();

                code = http.getResponseCode();
                response.setStatus(code);
            }

            if (proxy_in == null) {
                try {
                    proxy_in = connection.getInputStream();
                }
                catch (Exception e) {
                    proxy_in = http.getErrorStream();
                }
            }

            // clear response defaults.
            response.setHeader("Date", null);
            response.setHeader("Server", null);

            // set response headers
            int h = 0;
            String hdr = connection.getHeaderFieldKey(h);
            String val = connection.getHeaderField(h);
            while (hdr != null || val != null) {
                String lhdr = hdr != null ? hdr.toLowerCase() : null;
                if (hdr != null && val != null && !_DontProxyHeaders.contains(lhdr))
                    response.addHeader(hdr, val);

                h++;
                hdr = connection.getHeaderFieldKey(h);
                val = connection.getHeaderField(h);
            }
            response.addHeader("Via", "INFINIDAT Proxy Plugin");

            if (proxy_in != null) {
                copy(proxy_in, response.getOutputStream());
            }
            response.flushBuffer();
        }
    }


    /* ------------------------------------------------------------ */
    public void handleConnect(HttpServletRequest request,
                              HttpServletResponse response)
        throws IOException
    {
        String uri = request.getRequestURI();

        String port = "";
        String host = "";

        int c = uri.indexOf(':');
        if (c >= 0) {
            port = uri.substring(c + 1);
            host = uri.substring(0, c);
            if (host.indexOf('/') > 0) {
                host = host.substring(host.indexOf('/') + 1);
            }
        }

        InetSocketAddress inetAddress = new InetSocketAddress(host, Integer.parseInt(port));

        //if (isForbidden(HttpMessage.__SSL_SCHEME, addrPort.getHost(), addrPort.getPort(), false))
        //{
        //    sendForbid(request, response, uri);
        //}
        //else
        {
            InputStream in = request.getInputStream();
            OutputStream out = response.getOutputStream();

            Socket socket = new Socket(inetAddress.getAddress(), inetAddress.getPort());

            response.setStatus(200);
            response.setHeader("Connection", "close");
            response.flushBuffer();

            copy(socket.getInputStream(), out);
            copy(in, socket.getOutputStream());

            socket.close();
        }
    }

    /* (non-Javadoc)
     * @see javax.servlet.Servlet#getServletInfo()
     */
    public String getServletInfo() {
        return "Proxy Servlet";
    }

    /* (non-Javadoc)
     * @see javax.servlet.Servlet#destroy()
     */
    public void destroy() {
    }

    String _prefix = "/vsphere-client/infinidat/proxy";
    String _pluginKey = null;
    String _proxyTo = null;

    private void readProxySettings() {
        InputStream urlstream = getClass().getResourceAsStream("proxy_url.txt");
        BufferedReader reader = new BufferedReader(new InputStreamReader(urlstream));
        try {
            _pluginKey = reader.readLine();
            _prefix = "/vsphere-client/" + reader.readLine() + "/proxy";
        } catch (Exception e) {
        }
    }

    protected URL proxyHttpURL(final String scheme, final String serverName, int serverPort, final String uri) throws MalformedURLException {
        if (_prefix != null && !uri.startsWith(_prefix)) {
            return null;
        }

        if (_prefix != null) {
            return new URL(_proxyTo + uri.substring(_prefix.length()));
        }
        return new URL(_proxyTo + uri);
    }

    public int bufferSize = 2*8192;
    public int copy(InputStream in, OutputStream out)
            throws IOException {
        byte buffer[] = new byte[bufferSize];
        int len = bufferSize;
        int total_len = 0;

        while (true) {
            len = in.read(buffer, 0, bufferSize);
            if (len < 0)
                break;

            out.write(buffer, 0, len);
            out.flush();
            total_len += len;
        }
        return total_len;
    }

    @Override
    public ServletConfig getServletConfig() {
        return null;
    }

    @Override
    public void init(ServletConfig config) throws ServletException {
        readProxySettings();
    }

    private static final String SERVICE_INSTANCE = "ServiceInstance";

    /** object for access to all of the methods defined in the vSphere API */
    private static VimPortType _vimPort = initializeVimPort();

    private static VimPortType initializeVimPort() {
        // Static initialization is preferred because it takes a few seconds.
        VimService vimService = new VimService();
        return vimService.getVimPort();
    }

    /**
     * Get the ServiceContent for a given VC server.
     */
    private ServiceContent getServiceContent(String sessionCookie, String serviceUrl) {
        List<String> values = new ArrayList<String>();
        values.add("vmware_soap_session=" + sessionCookie);
        Map<String, List<String>> reqHeadrs = new HashMap<String, List<String>>();
        reqHeadrs.put("Cookie", values);

        Map<String, Object> reqContext = ((BindingProvider) _vimPort)
                .getRequestContext();
        reqContext.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, serviceUrl);
        reqContext.put(BindingProvider.SESSION_MAINTAIN_PROPERTY, true);
        reqContext.put(MessageContext.HTTP_REQUEST_HEADERS, reqHeadrs);

        final ManagedObjectReference svcInstanceRef = new ManagedObjectReference();
        svcInstanceRef.setType(SERVICE_INSTANCE);
        svcInstanceRef.setValue(SERVICE_INSTANCE);

        try {
            DisableSecurity.trustEveryone();
        } catch (KeyManagementException e) {
        } catch (NoSuchAlgorithmException e) {
        }

        ServiceContent serviceContent = null;
        try {
            serviceContent = _vimPort.retrieveServiceContent(svcInstanceRef);
        } catch (RuntimeFaultFaultMsg e) {
        }

        return serviceContent;
    }

    private String getPluginUrl(String sessionCookie, String serviceUrl) {
        ServiceContent service = getServiceContent(sessionCookie, serviceUrl);
        if (service != null) {
            try {
                Extension ext = _vimPort.findExtension(service.getExtensionManager(), _pluginKey);
                List<ExtensionServerInfo> server = ext.getServer();
                String fullUrl = server.get(0).getUrl();
                return fullUrl.replace("/plugin/info", "");
            }
            catch (Exception e) {
            }
        }
        return "http://localhost";
    }
}
