from infi.vendata.vmware_powertools.config import Configuration
import os
from logging import getLogger

logger = getLogger(__name__)


def sanitized_key():
    # converts keys like com.infinidat.powertools.develop.host-ci27 to com_infinidat_powertools_develop_host_ci27
    import re
    return re.sub("\W+", "_", Configuration.get_vcenter_extension_key())


def _write_proxy_file():
    # write two lines to proxy_url.txt - 1. the url to proxy 2. the uri prefix
    file_path = os.path.join(os.path.dirname(__file__),
        "war_src", "WEB-INF", "classes", "com", "infinidat", "proxyServlet", "proxy_url.txt")
    with open(file_path, "wb") as fd:
        fd.write("\n".join([Configuration.get_vcenter_extension_key(), sanitized_key()]))


def _write_template(filename, **template_args):
    from jinja2 import Template
    with open(filename + ".template") as fd:
        data = fd.read()
    Template(data).stream(**template_args).dump(filename)


def _write_files_from_templates():
    from infi.vendata.vmware_powertools import info

    file_path = os.path.join(os.path.dirname(__file__), "plugin", "plugin-package.xml")
    _write_template(file_path, key=Configuration.get_vcenter_extension_key(),
                               version=info.get_product_version(),
                               name=info.get_product_name(),
                               desc=info.get_product_description(),
                               company=info.get_company_name())

    file_path = os.path.join(os.path.dirname(__file__), "war_src", "plugin.xml")
    _write_template(file_path, key=Configuration.get_vcenter_extension_key(),
                               sanitized_key=sanitized_key())

    file_path = os.path.join(os.path.dirname(__file__), "war_src", "META-INF", "MANIFEST.MF")
    _write_template(file_path, key=Configuration.get_vcenter_extension_key(),
                               sanitized_key=sanitized_key())

    file_path = os.path.join(os.path.dirname(__file__), "war_src", "resources", "js", "web-platform.js")
    _write_template(file_path, key=Configuration.get_vcenter_extension_key(),
                               sanitized_key=sanitized_key())


def _create_zip_from_dir(dirname, target_zip_path):
    import zipfile
    if os.path.exists(target_zip_path):
        os.remove(target_zip_path)
    if not os.path.exists(os.path.dirname(target_zip_path)):
        os.makedirs(os.path.dirname(target_zip_path))
    zip_file = zipfile.ZipFile(target_zip_path, "w", zipfile.ZIP_DEFLATED)
    for p, ds, fs in os.walk(dirname):
        for f in fs:
            zip_file.write(os.path.join(p, f), os.path.join(p, f).replace(dirname + os.path.sep, ""))
    zip_file.close()


def _create_war_from_src():
    war_path = os.path.join(os.path.dirname(__file__), "plugin", "plugins", "infinidat.war")
    dirname = os.path.join(os.path.dirname(__file__), "war_src")
    _create_zip_from_dir(dirname, war_path)


def _create_zip_from_plugin():
    zip_path = os.path.join(os.path.dirname(__file__), "plugin", "vwc.zip")
    dirname = os.path.join(os.path.dirname(__file__), "plugin")
    _create_zip_from_dir(dirname, zip_path)


def _update_extension():
    from infi.vendata.vmware_powertools.vcenter import vCenter
    from infi.vendata.vmware_powertools import info
    from pyVmomi import vim
    desc = vim.Description(label=info.get_product_description(), summary=info.get_product_name())
    url = "{}/vwc/vwc.zip".format(Configuration.get_webapp_ssl_url())
    extension_client_info = vim.ExtensionClientInfo(
        description=desc,
        version=info.get_product_version(),
        company=info.get_company_name(),
        type="vsphere-client-serenity",
        url=url
    )
    client = vCenter.get_extension_client()
    extension_manager = client.service_content.extensionManager
    extension = extension_manager.FindExtension(Configuration.get_vcenter_extension_key())
    extension.client = [extension_client_info]
    extension_manager.UpdateExtension(extension)


def register_vwc():
    logger.info("Registering web client")
    _write_proxy_file()
    _write_files_from_templates()
    _create_war_from_src()
    _create_zip_from_plugin()
    _update_extension()
    logger.info("Web client registered")
