from infi.vendata.vmware_powertools.webapp import json_response
from infi.vendata.vmware_powertools.utils import auth
from infi.credentials_store import Credentials
from infi.vendata.vmware_powertools.patches import with_host

def isbool(val):
    return val in (True, "true")

class Auth(object):

    def get_host_by_moref(self, moref, serial):
        from infi.vendata.vmware_powertools.vcenter.shortcuts import moref_to_object, get_datacenter_clusters
        from infi.vendata.vmware_powertools.snapshots.vendor_query import VendorQuery
        from pyVmomi import vim
        mo = moref_to_object(moref)
        if isinstance(mo, vim.HostSystem):
            return mo
        if isinstance(mo, vim.Datacenter):
            return get_datacenter_clusters(mo)[0].host[0]
        if isinstance(mo, vim.VirtualMachine):
            return mo.datastore[0].host[0].key
        if isinstance(mo, vim.Datastore):
            return mo.host[0].key
        if isinstance(mo, vim.ClusterComputeResource):
            return mo.host[0]
        if isinstance(mo, vim.Folder):
            for vm in mo.childEntity:
                if not isinstance(vm, vim.VirtualMachine):
                    continue
                vm_system_serial = VendorQuery.get_system_serial_from_datastore(vm)
                if vm_system_serial is not None and vm_system_serial == int(serial):
                    return vm.datastore[0].host[0].key
        raise Exception("No relevant ESX host found")

    @json_response
    def has_credentials(self, serial, moref, **kwargs):
        host = self.get_host_by_moref(moref, serial)
        with with_host(host):
            if auth.auth.CREDENTIALS_STORE.get_existing_credentials(serial):
                return True
            return False

    @json_response
    def set_credentials(self, serial, username, password, remember_me, default, moref, **kwargs):
        host = self.get_host_by_moref(moref, serial)
        with with_host(host):
            credentials = Credentials(username, password)
            if auth.auth.CREDENTIALS_STORE.authenticate(serial, credentials):
                if isbool(default):
                    auth.auth.CREDENTIALS_STORE.set_default_credentials(credentials, remember_me=isbool(remember_me))
                else:
                    auth.auth.CREDENTIALS_STORE.set_credentials(serial, credentials, remember_me=isbool(remember_me))
                return True
            return False

    @json_response
    def reset_credentials(self, **kwargs):
        auth.auth.CREDENTIALS_STORE.reset_credentials()

    @json_response
    def get_current_username(self, serial, moref, **kwargs):
        host = self.get_host_by_moref(moref, serial)
        with with_host(host):
            credentials = auth.auth.CREDENTIALS_STORE.get_existing_credentials(serial)
            return None if credentials is None else credentials.get_username()
