import cherrypy
import logging
from ...host.datastores import DatastoresResource
from infi.vendata.powertools.gui.webapp.plugin.views.host.volumes import BoxVolumes
from infi.vendata.vmware_powertools.vcenter.shortcuts import moref_to_object, object_to_moref
from infi.vendata.vmware_powertools.mapping import good_cluster_hosts

logger = logging.getLogger(__name__)


class ClusterDatastoresTable(object):
    COLUMN_NAMES = ["Datastore Name", "Volume Name", "Size", "# Hosts", "Lun", "Device Name", "Actions"]

    def __init__(self, cluster):
        super(ClusterDatastoresTable, self).__init__()
        self.cluster = cluster

    def get_row_data(self, system_serial):
        rows = dict()  # from device serial to row data
        get_host_row_data = cherrypy.tree.apps[''].root.plugin.views.host.datastores.get_row_data
        for esx_host in good_cluster_hosts(self.cluster):
            host_ref = object_to_moref(esx_host)
            for item in get_host_row_data(host_ref, system_serial):
                device_serial = item['serial']
                if device_serial in rows:
                    if not rows[device_serial]['reclaim_available']:
                        rows[device_serial]['reclaim_available'] = item['reclaim_available']
                    rows[device_serial]['hosts'] += 1
                else:
                    item.pop('paths', None)
                    item['hosts'] = 1
                    rows[device_serial] = item
        return rows.values()


class ClusterDatastores(DatastoresResource):
    def _get_table_builder(self, moref):
        cluster = moref_to_object(moref)
        return ClusterDatastoresTable(cluster)


class ClusterBoxVolumes(BoxVolumes):

    @cherrypy.expose
    def index(self, moref, serial, **kwargs):
        from infi.vendata.vmware_powertools.patches import with_host
        self._cluster = moref_to_object(moref)
        for esx_host in good_cluster_hosts(self._cluster):
            with with_host(esx_host):
                # we return the index for the first host, it should be correct for all hosts
                return super(ClusterBoxVolumes, self).index(serial, show_mapped=True, **kwargs)
