from infi.vendata.vmware_powertools.utils import spawn, joinall
from infi.vendata.vmware_powertools.vcenter.shortcuts import create_vcenter_task
from infi.vendata.vmware_powertools.mapping import good_cluster_hosts


class ClusterMapVolume(object):
    def _mount_datastore(self):
        self._task.set_description("Mounting datastore")
        joinall([spawn(operation.mount_datastore) for operation in self._operations], raise_error=True)

    def _map_volume(self):
        from infi.vendata.vmware_powertools.utils import refresh_storage_model_on_cluster
        self._task.set_description("Mapping volume to cluster")
        self._operations[0].map_to_cluster()                                      # map to cluster only once
        refresh_storage_model_on_cluster(self._esx_cluster)                       # but rescan on all hosts

    def run(self, esx_cluster, system_serial, volume_id, do_mount=True):
        from infi.vendata.vmware_powertools.vcenter.map import MapVolume
        self._task = create_vcenter_task(esx_cluster, "Volume map")
        self._esx_cluster = esx_cluster
        self._operations = [MapVolume(esx_host, int(system_serial), int(volume_id))
                            for esx_host in good_cluster_hosts(esx_cluster, maintenance_hosts=False)]
        steps = [self._map_volume, self._mount_datastore] if do_mount else [self._map_volume]
        self._task.step(steps)
