from infi.vendata.vmware_powertools.utils import spawn, joinall
from infi.vendata.vmware_powertools.vcenter.shortcuts import create_vcenter_task
from infi.vendata.vmware_powertools.patches import with_host
from infi.vendata.vmware_powertools.mapping import good_cluster_hosts


class ClusterUnmapVolume(object):
    def __init__(self, esx_cluster, proper_serial):
        from infi.vendata.vmware_powertools.vcenter.unmap import VolumeUnmapBase
        self._esx_cluster = esx_cluster
        self._proper_serial = proper_serial
        self._operations = [VolumeUnmapBase(esx_host, proper_serial, True) for esx_host in good_cluster_hosts(esx_cluster, maintenance_hosts=False)]
        self._task = create_vcenter_task(esx_cluster, "Volume unmap")

    def _assert_cluster_mapping(self):
        from infi.vendata.vmware_powertools.utils import get_infinidat_device_by_serial
        from infi.vendata.vmware_powertools.mapping import is_cluster_mapped
        from infi.vendata.vmware_powertools.vcenter.unmap import ValidationError
        device = None
        for host in good_cluster_hosts(self._esx_cluster):
            device, matching_host = get_infinidat_device_by_serial(host, self._proper_serial), host
            if device is not None:
                break
        if device is None:
            raise ValidationError("Could not find a storage device matching the SCSI serial for this volume: {}".format(self._proper_serial))
        with with_host(matching_host):
            if not is_cluster_mapped(device):
                raise ValidationError("The volume is not mapped directly to this cluster. It cannot be unmapped from this view.")

    def _check_assertions(self):
        self._task.set_description("Checking if lun can be safely detached")
        self._assert_cluster_mapping()
        joinall([spawn(operation.assert_safe_to_remove) for operation in self._operations], raise_error=True)

    def _unmount_datastore(self):
        self._task.set_description("Unmounting and detaching")
        joinall([spawn(operation.unmount_datastore) for operation in self._operations], raise_error=True)
        joinall([spawn(operation.detach) for operation in self._operations], raise_error=True)

    def _unmap(self):
        from infi.vendata.vmware_powertools.utils import refresh_storage_model_on_cluster
        self._task.set_description("Unmapping volume")
        self._operations[0].unmap()
        refresh_storage_model_on_cluster(self._esx_cluster)

    def _delete(self):
        self._task.set_description("Deleting volume")
        self._operations[0].delete()

    def unmap(self):
        steps = [self._check_assertions, self._unmount_datastore, self._unmap]
        self._task.step(steps)

    def delete(self):
        steps = [self._check_assertions, self._unmount_datastore, self._unmap, self._delete]
        self._task.step(steps)
