import cherrypy
from ....resource import PluginResource
from infi.vendata.vmware_powertools.webapp import json_response
from infi.vendata.vmware_powertools.mapping import good_cluster_hosts
from infi.vendata.vmware_powertools.mapping import NoHostsException

class Readiness(PluginResource):
    @json_response
    def index(self, moref, silent, **kwargs):
        return getattr(self, cherrypy.request.method.upper())(moref, silent)

    def POST(self, moref, silent):
        return self.do(moref, silent, 'POST')

    def GET(self, moref, silent):
        return self.do(moref, silent, 'GET')

    def do(self, moref, silent, method_name):
        from infi.vendata.vmware_powertools.vcenter.shortcuts import moref_to_object
        from infi.vendata.vmware_powertools.vcenter import vCenter
        from infi.vendata.vmware_powertools.utils import spawn, joinall
        client = vCenter.get_extension_client()
        host_readiness = cherrypy.tree.apps[''].root.plugin.views.host.readiness
        esx_cluster = moref_to_object(moref)
        method = getattr(host_readiness, method_name)
        try:
            greenlets = [spawn(method, client.get_reference_to_managed_object(esx_host), silent)
                         for esx_host in good_cluster_hosts(esx_cluster, maintenance_hosts=False)]
        except NoHostsException:
            # no hosts - no worries
            return True
        joinall(greenlets)
        results = [green.get() for green in greenlets]
        error_messages = [error_message for error_message in results if error_message is not True]
        if len(error_messages) > 0:
            return error_messages[0]
        return True
