import cherrypy
import logging
from infi.vendata.vmware_powertools.webapp import json_response
from infi.vendata.vmware_powertools.vcenter.shortcuts import moref_to_object, object_to_moref
from infi.vendata.vmware_powertools.mapping import good_cluster_hosts

from ....resource import PluginResource

logger = logging.getLogger(__name__)

def register_cluster(esx_cluster, system_serial):
    from .engine import VMwareClusterRegistration
    engine = VMwareClusterRegistration(esx_cluster, system_serial)
    engine.register_cluster()

def _get_storage_controller_for_system(system_serial):
    from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_block_devices_and_controllers__mapped_to_lun0
    device = [device for device in get_infinidat_block_devices_and_controllers__mapped_to_lun0()
              if device.get_vendor().get_system_serial() == system_serial]
    return None if not device else device[0]

def _is_device_does_not_belong_to_a_cluster_host(device):
    return device is None or device.get_vendor().get_cluster_id() in [0, -1]

def is_cluster_registered_in_system(esx_cluster, system_serial):
    from infi.vendata.vmware_powertools.patches import with_host
    clusters = set()
    for esx_host in good_cluster_hosts(esx_cluster):
        with with_host(esx_host):
            device = _get_storage_controller_for_system(system_serial)
            if _is_device_does_not_belong_to_a_cluster_host(device):
                logger.debug("Host {} does not belong to cluster".format(esx_host.summary.config.name))
                return False
            vendor = device.get_vendor()
            clusters.add(vendor.get_cluster_id())
            logger.debug("Host {} belongs to cluster {}".format(esx_host.summary.config.name, vendor.get_cluster_id()))
    return len(clusters) == 1 and 0 not in clusters and -1 not in clusters

class ClusterRegistration(PluginResource):
    @json_response
    def index(self, moref, serial, **kwargs):
        method = getattr(self, cherrypy.request.method.upper())
        return method(moref, int(serial))

    def POST(self, moref, serial):
        esx_cluster = moref_to_object(moref)
        register_cluster(esx_cluster, serial)
        self.set_cached_entry(True, self.GET, moref, serial)
        host_registration = cherrypy.tree.apps[''].root.plugin.views.host.registration
        for esx_host in good_cluster_hosts(esx_cluster):
            host_moref = object_to_moref(esx_host)
            self.set_cached_entry(True, host_registration.GET, host_moref, serial)

    @PluginResource.cached_method
    def GET(self, moref, serial):
        esx_cluster = moref_to_object(moref)
        response = is_cluster_registered_in_system(esx_cluster, serial)
        return response
