from logging import getLogger
logger = getLogger(__name__)

def get_inquiry_mixin_for_system_by_serial(system_serial):
    from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_block_devices_and_controllers__mapped_to_lun0
    for device in get_infinidat_block_devices_and_controllers__mapped_to_lun0():
        if device.get_vendor().get_system_serial() == system_serial:
            return device.get_vendor()

def get_infinisdk_system_object_from_inquiry(inquiry_mixin):
    from infi.vendata.vmware_powertools.utils import get_system_object_by_device_vendor
    return get_system_object_by_device_vendor(inquiry_mixin)

def get_host_object_in_system(system, inquiry_mixin):
    return system.hosts.get(id=inquiry_mixin.get_host_id())

def is_host_part_of_a_cluster(host_object):
    return host_object.get_cluster() is not None

def get_cluster_object_from_inquiry(system, inquiry_mixin):
    return system.host_clusters.get(id=inquiry_mixin.get_cluster_id())

def is_cluster_name_matches(cluster_object, expected_cluster_name):
    return cluster_object.get_name() == expected_cluster_name

def get_all_host_objects(system_object, cluster_object, esx_cluster):
    from infi.vendata.vmware_powertools.patches import with_host
    from infinisdk.core.exceptions import ObjectNotFound
    from infi.vendata.vmware_powertools.mapping import good_cluster_hosts
    # this function is called after we regisetred all the hosts, so we can't just look their objects by name
    host_objects = set()
    for esx_host in good_cluster_hosts(esx_cluster):
        try:
            host_objects.add(system_object.hosts.get(name=esx_host.summary.config.name))
        except ObjectNotFound:
            with with_host(esx_host):
                inquiry_mixin = get_inquiry_mixin_for_system_by_serial(system_object.get_serial())
                host_objects.add(get_host_object_in_system(system_object, inquiry_mixin))
    return host_objects

def are_there_foreign_hosts_in_cluster(system_object, cluster_object, esx_cluster):
    host_objects = get_all_host_objects(system_object, cluster_object, esx_cluster)
    difference = set(cluster_object.get_hosts()).difference(host_objects)
    logger.debug("are_there_foreign_hosts_in_cluster {!r} {!r}".format(host_objects, cluster_object.get_hosts()))
    return difference

def are_there_volumes_mapped_to_cluster(cluster_object):
    return bool([lun_mapping for lun_mapping in cluster_object.get_luns()
                 if lun_mapping.clustered])

def is_cluster_empty_of_hosts(cluster_object):
    return len(cluster_object.get_hosts()) == 0

def is_cluster_object_exists(system_object, expected_cluster_name):
    return get_cluster_object_by_name(system_object, expected_cluster_name) is not None

def get_cluster_object_by_name(system_object, expected_cluster_name):
    return system_object.host_clusters.safe_get(name=expected_cluster_name)

def find_next_available_name(system, base_name):
    from itertools import count
    for index in count(1):
        name = '{}-{}'.format(base_name, index)
        obj = get_cluster_object_by_name(system, name)
        if obj is None:
            return name
