from infi.vendata.vmware_powertools.webapp.plugin.resource import PluginResource
from .rename import HostRenameVolume
from .unmap import HostUnmapVolume
from .map import HostMapVolume
from .resize import Resize
from .reclaim import reclaim, BadEsxVersion, esx_supports_reclaim
from .provision import Provisioning
from .utils import create_vmfs_datastore_by_serial
from .utils import host_datastores_changed
from .utils import fix_serial, get_system_serial
from infi.vendata.vmware_powertools.webapp import json_response
from infi.vendata.vmware_powertools.vcenter.shortcuts import moref_to_object
from infi.vendata.vmware_powertools.patches import with_host


class Operations(PluginResource):
    @json_response
    def map_existing(self, moref, serial, volume_id, **kwargs):
        host = moref_to_object(moref)
        HostMapVolume().map(host, serial, volume_id)
        host_datastores_changed(moref, int(serial))

    @json_response
    def query_mount_after_map(self, serial, volume_id, **kwargs):
        return dict(has_fs=False)

    @json_response
    def unmap(self, moref, serial, **kwargs):
        from infi.storagemodel import get_storage_model
        host = moref_to_object(moref)
        proper_serial = fix_serial(serial)
        HostUnmapVolume(host, proper_serial).unmap()
        with with_host(host):
            get_storage_model().refresh()
        host_datastores_changed(moref, get_system_serial(serial))

    @json_response
    def delete_vol(self, moref, serial, **kwargs):
        from infi.storagemodel import get_storage_model
        host = moref_to_object(moref)
        proper_serial = fix_serial(serial)
        HostUnmapVolume(host, proper_serial).delete()
        with with_host(host):
            get_storage_model().refresh()
        host_datastores_changed(moref, get_system_serial(serial))

    @json_response
    def rescan(self, moref, **kwargs):
        from infi.vendata.vmware_powertools.utils import refresh_storage_model_on_host
        refresh_storage_model_on_host(moref_to_object(moref), raise_error=True, do_rescan=True, do_refresh=True)

    @json_response
    def rename(self, moref, serial, name, rename_datastore, rename_device, **kwargs):
        host = moref_to_object(moref)
        proper_serial = fix_serial(serial)
        HostRenameVolume().rename(host, proper_serial, name, rename_datastore, rename_device)

    @json_response
    def reclaim(self, moref, serial, name, **kwargs):
        host = moref_to_object(moref)
        if not esx_supports_reclaim(host):
            raise BadEsxVersion("ESX version does not support Reclaim Space operation")
        proper_serial = fix_serial(serial)
        result = reclaim(host, proper_serial, name)
        host_datastores_changed(moref, get_system_serial(serial))
        return dict(reclaimed=result)

    @json_response
    def create_filesystem(self, moref, serial, **kwargs):
        host = moref_to_object(moref)
        proper_serial = fix_serial(serial)
        create_vmfs_datastore_by_serial(host, proper_serial)

    @json_response
    def update_metadata(self, moref, **kwargs):
        from infi.vendata.vmware_powertools.patches.powertools_patches import set_esx_host_in_local_storage
        from infi.vendata.powertools.metadata_sync.engine import metadata_sync
        host = moref_to_object(moref)
        set_esx_host_in_local_storage(host)
        with with_host(host):
            metadata_sync(silent=True)

    resize = Resize()
    provisioning = Provisioning()
