from infi.vendata.vmware_powertools.utils import get_infinidat_device_by_serial
from infi.vendata.vmware_powertools.utils import get_infinisdk_volume_by_device
from infi.vendata.vmware_powertools.patches import with_host


MIN_ESX_VERSION_FOR_RECLAIM = "5.5.0"


class BadEsxVersion(Exception):
    pass


def esx_supports_reclaim(esx_host):
    from pkg_resources import parse_version
    return parse_version(esx_host.summary.config.product.version) >= parse_version(MIN_ESX_VERSION_FOR_RECLAIM)


def reclaim(host, serial, name):
    from infi.pyvmomi_wrapper.esxcli import EsxCLI
    device = get_infinidat_device_by_serial(host, serial)
    with with_host(host):
        volume = get_infinisdk_volume_by_device(device)
    pre_allocated = volume.get_allocated()
    cli = EsxCLI(host)
    vmfs = cli.get("storage.vmfs")
    vmfs.Unmap(reclaimunit=None, volumelabel=name)
    volume.refresh()
    post_allocated = volume.get_allocated()
    delta = pre_allocated - post_allocated
    return "0 bytes" if delta.bits <= 0 else repr(delta).replace("*", " ")
