from infi.vendata.vmware_powertools.utils import get_infinisdk_volume_by_device
from infi.vendata.vmware_powertools.mapping import get_all_host_mappings
from infi.vendata.vmware_powertools.utils import spawn, joinall
from infi.vendata.vmware_powertools.utils import get_infinidat_device_by_serial
from infi.vendata.vmware_powertools.patches import with_host
from infi.vendata.vmware_powertools.vcenter.shortcuts import wait_for_task
from .utils import datastore_changed
from .utils import update_device_name, get_datastore_by_serial_number


class HostRenameVolume(object):
    def _wait_for_volume_change_on_host(self, host, volume_serial, expected_name):
        from infi.storagemodel import get_storage_model
        from infi.vendata.vmware_powertools.utils import get_infinidat_device_by_serial__inner
        with with_host(host):
            def _name_changed_predicate():
                dev = get_infinidat_device_by_serial__inner(volume_serial)
                return True if dev is None else dev.get_vendor().get_volume_name() == expected_name
            get_storage_model().rescan_and_wait_for(_name_changed_predicate, do_rescan=False, do_refresh=True)

    def _wait_for_volume_name_change(self, volume_serial, expected_name):
        all_hosts = get_all_host_mappings(volume_serial)
        joinall([spawn(self._wait_for_volume_change_on_host, host, volume_serial, expected_name)
                 for host in all_hosts], raise_error=True)

    def _update_name_for_device_with_serial_number(self, host, serial_number):
        with with_host(host):
            from infi.storagemodel.vendor.infinidat.shortcuts import get_infinidat_block_devices_and_controllers
            from infi.storagemodel import get_storage_model
            for device in [device for device in get_infinidat_block_devices_and_controllers()
                           if device.get_scsi_serial_number() == serial_number]:
                updated = update_device_name(host, device)
                if updated:
                    get_storage_model().refresh()
                    datastore_changed(serial_number)

    def rename(self, host, proper_serial, name, rename_datastore, rename_device):
        from infi.vendata.powertools.cli.validate import validate_name
        validate_name(name)
        datastore = get_datastore_by_serial_number(host, proper_serial)
        device = get_infinidat_device_by_serial(host, proper_serial)
        with with_host(host):
            volume = get_infinisdk_volume_by_device(device)
        old_name = volume.get_name()
        # Rename volume
        if name != old_name:
            volume.update_name(name)
            self._wait_for_volume_name_change(proper_serial, name)
        # Rename datastore
        if datastore and rename_datastore and datastore.name != name:
            task = datastore.Rename_Task(name)
            wait_for_task(task)
        # Rename device
        if rename_device:
            self._update_name_for_device_with_serial_number(host, proper_serial)
        # Refresh
        datastore_changed(proper_serial)
