from infi.vendata.vmware_powertools.utils import get_infinidat_device_by_serial
from infi.vendata.vmware_powertools.vcenter.shortcuts import create_vcenter_task
from infi.vendata.vmware_powertools.patches import with_host


class HostUnmapVolume(object):
    def __init__(self, esx_host, proper_serial):
        from infi.vendata.vmware_powertools.vcenter.unmap import VolumeUnmapBase
        self._esx_host = esx_host
        self._proper_serial = proper_serial
        self._task = create_vcenter_task(esx_host, "Volume unmap")
        self._operation = VolumeUnmapBase(esx_host, proper_serial)

    def _check_assertions(self):
        from infi.vendata.vmware_powertools.mapping import is_cluster_mapped
        from infi.vendata.vmware_powertools.vcenter.unmap import ValidationError
        self._task.set_description("Checking if lun can be safely detached")
        device = get_infinidat_device_by_serial(self._esx_host, self._proper_serial)
        with with_host(self._esx_host):
            if is_cluster_mapped(device):
                raise ValidationError("The volume is not mapped directly to this host. It cannot be unmapped from this view.")
        self._operation.assert_safe_to_remove()

    def _detach(self):
        self._task.set_description("Unmounting and detaching")
        self._operation.unmount_datastore()
        self._operation.detach()

    def _unmap(self):
        from infi.vendata.vmware_powertools.utils import refresh_storage_model_on_host
        self._task.set_description("Unmapping volume")
        self._operation.unmap()
        refresh_storage_model_on_host(self._esx_host)

    def _delete(self):
        self._task.set_description("Deleting volume")
        self._operation.delete()

    def unmap(self):
        steps = [self._check_assertions, self._detach, self._unmap]
        self._task.step(steps)

    def force_unmap(self):
        # used by commvault
        steps = [self._unmap]
        self._task.step(steps)

    def delete(self):
        steps = [self._check_assertions, self._detach, self._unmap, self._delete]
        self._task.step(steps)
