import cherrypy
import logging
from infi.vendata.vmware_powertools.vcenter.shortcuts import moref_to_object

from ....resource import PluginResource
from infi.vendata.vmware_powertools.webapp import json_response

logger = logging.getLogger(__name__)

def get_infinisdk_and_local_fc_ports_set_and_device(esx_host, system_serial):
    from infi.vendata.vmware_powertools.patches import with_host
    from infi.vendata.vmware_powertools.vcenter.map import HostSystemByController
    with with_host(esx_host):
        from infi.hbaapi import get_ports_collection
        ports = set([port.port_wwn for port in get_ports_collection().get_ports()])
        host_system = HostSystemByController(esx_host, system_serial)
    system = host_system.get_system()
    device = host_system.get_controller_device()
    return system, ports, device

def register_host(esx_host, system_serial):
    from ..operations.utils import update_device_name
    from infi.vendata.vmware_powertools.patches import with_host
    from infi.vendata.powertools.gui.webapp.plugin.views.host import registration
    from infi.vendata.powertools.system_register.engine import Abort
    from infi.vendata.vmware_powertools.patches.powertools_patches import set_esx_host_in_local_storage
    set_esx_host_in_local_storage(esx_host)
    system, ports, device = get_infinisdk_and_local_fc_ports_set_and_device(esx_host, system_serial)
    with with_host(esx_host):
        try:
            registration.register_host(system_serial)
            update_device_name(esx_host, device)
        except Abort as error:  # pragma: no cover
            raise cherrypy.HTTPError(409, error.message)

def is_host_registration_supported(esx_host, system_serial):
    from infi.vendata.powertools.utils.versioncheck import UnsupportedVersion
    try:
        system, ports, _ = get_infinisdk_and_local_fc_ports_set_and_device(esx_host, system_serial)
    except UnsupportedVersion:
        return False
    return True

def is_host_registered_in_system(esx_host, system_serial):
    from infi.vendata.vmware_powertools.patches import with_host
    from infi.vendata.powertools.gui.webapp.plugin.views.host import registration
    with with_host(esx_host):
        return registration.is_host_registered_in_system(system_serial)

class HostRegistration(PluginResource):
    @json_response
    def index(self, moref, serial, **kwargs):
        method = getattr(self, cherrypy.request.method.upper())
        return method(moref, int(serial))

    def POST(self, moref, serial):
        esx_host = moref_to_object(moref)
        register_host(esx_host, serial)
        self.set_cached_entry(True, self.GET, moref, serial)

    @PluginResource.cached_method
    def GET(self, moref, serial):
        esx_host = moref_to_object(moref)
        response = is_host_registered_in_system(esx_host, serial)
        return response
