var operations = {
    updateTableCallback: function() {
        var dataTable = $("#snapshot_table").dataTable();
        virtualmachine.loadData();
    },

    checkGoBack: function(json) {
        if (json.return_value != null && json.return_value["go_back"] == true) {
            if ($("#back").length > 0) {
                $("#back")[0].click();
            }
        }
    },

    revert_snapshot: function(event) {
        var selElement = event.target;
        var actionsElement = $(selElement).closest('.actions');
        var serial = actionsElement.data("infinibox-serial");
        auth.spin_call_with_credentials(serial, function() {
            var selElement = event.target;
            var data = {
                snapshot_id: actionsElement.data('snapshot-id'),
                expose_only: $(selElement).hasClass("expose_snapshot"),
            };
            var ajaxOptions = {type: "POST", dataType: "json", data: data, cache: false, url: sources.operations.get_revert_snapshot_parameters};
            view.spinAjax(ajaxOptions, operations.revert_snapshot_popup);
        });
    },

    revert_snapshot_popup: function(json) {
        var expose_only = json.return_value.expose_only;
        $("#snap-modal-name").val(json.return_value.name);
        $.each(["host", "datastore", "folder", "datacenter"], function(i, obj_name) {
            var obj_from_json = json.return_value[obj_name];
            var name = obj_from_json === null ? "" : obj_from_json.name;
            var moref = obj_from_json === null ? "" : obj_from_json.moref;
            $("#snap-modal-" + obj_name).val(name);
            $("#snap-modal-" + obj_name).data("moref", moref);
        })
        var elem = $("#revert-snap-modal");
        elem.dialog({
            modal: true,
            width: 560,
            height: 380,
            resizable: false,
            buttons: {
                DONE: function() {
                    var data = {
                        snapshot_id: json.return_value.snapshot_id,
                        vm_name: $("#snap-modal-name", elem).val().trim(),
                        host: $("#snap-modal-host", elem).data("moref"),
                        datastore: $("#snap-modal-datastore", elem).data("moref"),
                        folder: $("#snap-modal-folder", elem).data("moref"),
                        expose_only: expose_only,
                    };
                    if (!data.vm_name || !data.host || !data.folder || (!data.datastore && !expose_only)) {
                        showError("ERROR", "ERROR: Missing parameters");
                        return;
                    }
                    var ajaxOptions = {type: "POST", dataType: "json", data: data, cache: false, url: sources.operations.revert_snapshot};
                    view.spinAjax(ajaxOptions, operations.revert_snapshot_done);
                },
                CANCEL: function() {
                    $(this).dialog("close");
                }
            }
        });
        expose_only ? $("#snap-modal-datastore-row").hide() : $("#snap-modal-datastore-row").show();
        dialog_title = expose_only ? "EXPOSE SNAPSHOT" : "RESTORE SNAPSHOT";
        $("#revert-snap-modal").siblings(".ui-dialog-titlebar").children(".ui-dialog-title").text(dialog_title);
        $("label[for='snap-modal-name']").text(expose_only ? "EXPOSED VM NAME" : "RESTORED VM NAME");
    },

    build_treeview: function(toplevel_moref, motype_name) {
        $("#snap-tree").fancytree({
            source: {
                url: sources.operations.get_tree_data + "&toplevel=True&tree_moref=" + toplevel_moref + "&motype_name=" + motype_name
            },
            autoScroll: true,
            lazyLoad: function(event, data)  {
                data.result = $.ajax({
                    url: sources.operations.get_tree_data + "&tree_moref=" + data.node.data.moref + "&motype_name=" + motype_name,
                    dataType: "json"
                });
            },
            beforeActivate: function(event, data) {
              if (data.node.data.freespace) {
                $("#snap-tree-note").text("Free space on datastore: " + data.node.data.freespace);
              } else {
                $("#snap-tree-note").text("");
              }
              if (!data.node.data.activatable) {
                operations.set_treeview_button("disable");
                if (data.tree.activeNode !== null) {
                    data.tree.activeNode.setActive(false);
                }
                return false;
              } else {
                operations.set_treeview_button("enable");
              }
            }
        });
    },

    set_treeview_button: function(new_mode) {
        $(".ui-dialog-buttonpane button:contains('DONE')").button(new_mode);
    },

    open_target_treeview: function(event) {
        $(":ui-fancytree").fancytree("destroy");
        var elem;
        var clicked_button_id = $(".modal-button", $(event.target).parent()).attr("id");
        if (clicked_button_id == "snap-host-btn") {
            elem = $("#snap-modal-host");
            operations.build_treeview($("#snap-modal-datacenter").data("moref"), "HostSystem");
        }
        else if (clicked_button_id == "snap-datastore-btn") {
            elem = $("#snap-modal-datastore");
            var host_moref = $("#snap-modal-host").data("moref");
            if (!host_moref) {
                return;
            }
            operations.build_treeview(host_moref, "Datastore");
        }
        else if (clicked_button_id == "snap-folder-btn") {
            elem = $("#snap-modal-folder");
            operations.build_treeview($("#snap-modal-datacenter").data("moref"), "Folder");
        }
        $("#snap-tree-note").text("");
        $("#snap-tree-modal").dialog({
            modal: true,
            width: 440,
            height: 365,
            resizable: false,
            buttons: {
                DONE: function() {
                    operations.set_treeview_button("enable");
                    $(this).dialog("close");
                    node = $("#snap-tree").fancytree("getActiveNode");
                    if (node) {
                        elem.val(node.title);
                        elem.data("moref", node.data.moref);
                        if (elem.attr("id") == "snap-modal-host") {
                            // when host changes, datastore becomes invalid
                            $("#snap-modal-datastore").val("");
                            $("#snap-modal-datastore").data("moref", "");
                        }
                    }
                },
                CANCEL: function() {
                    operations.set_treeview_button("enable");
                    $(this).dialog("close");
                }
            }});
        operations.set_treeview_button("disable");
    },

    revert_snapshot_done: function(json) {
        $("#revert-snap-modal").dialog("close");
        var op = json.return_value.new_datastore_name !== null ? "exposed" : "restored";
        var title = "SNAPSHOT " + op.toUpperCase();
        var msgs = [];
        msgs.push("The snapshot was successfully " + op + " as virtual machine '" + json.return_value.vm_name + "'");
        if (json.return_value.cloned_vol_name !== null) {
            msgs.push("The new volume '" + json.return_value.cloned_vol_name + "' was created and mapped ");
        }
        if (json.return_value.new_datastore_name !== null) {
            msgs.push("The datastore '" + json.return_value.new_datastore_name + "' was mounted.");
        }

        var callback = function() { operations.checkGoBack(json); };
        view.show_summary_dialog(title, msgs, callback);
    },

    removeSnapshotDoneCallback: function(json) {
        operations.updateTableCallback();
        operations.checkGoBack(json);
    },

    confirm_remove_snapshot: function(event) {
        var selElement = event.target;
        var actionsElement = $(selElement).closest('.actions');
        var serial = actionsElement.data("infinibox-serial");
        auth.spin_call_with_credentials(serial, function() {
            var ajaxSource = sources.operations.confirm_remove_snapshot;
            var data = {snapshot_id: actionsElement.data('snapshot-id')};
            var ajaxOptions = {type: "POST", dataType: "json", data: data, cache: false, url: ajaxSource};
            var removeAjaxSource = ajaxSource.replace("confirm_", "");
            var confirmResultCallback = operations.confirm_remove_snapshot_result(removeAjaxSource, data);
            view.spinAjax(ajaxOptions, confirmResultCallback);
        });
    },

    confirm_remove_snapshot_result: function(ajaxSource, data) {
        var callback = function(json) {
            message = json.return_value[0];
            description = json.return_value[1];
            show_areyousure("DELETE SNAPSHOT", message, description, 'DELETE', function() {
                operations.remove_snapshot(ajaxSource, data);
            });
        }
        return callback;
    },

    remove_snapshot: function(ajaxSource, data) {
        var ajaxOptions = {type: "POST", dataType: "json", data: data, cache: false, url: ajaxSource};
        view.spinAjax(ajaxOptions, operations.removeSnapshotDoneCallback);
    },

    get_default_infinibox_alias: function() {
        var result = "vm_snap_";
        for (i = 0; i < 16; i ++) {
            result += Math.floor(Math.random()*16).toString(16);
        }
        return result;
    },

    create_snapshot_popup: function(event) {
        var serial = $(".infinibox.table").data("serial");
        auth.spin_call_with_credentials(serial, function() {
            var modal = $("#create-snap-modal");
            $(".snapname", modal).val(strftime("Snapshot %Y-%m-%d %Hh%Mm%Ss"));
            name = $(".moref").text();
            $(".snapdesc", modal).val("Virtual machines in this snapshot:\n  " + name);
            $(".snapobjname", modal).val(operations.get_default_infinibox_alias());
            modal.dialog({
                modal: true,
                width: 540,
                height: 540,
                resizable: false,
                buttons: {
                    DONE: function() {
                        operations.create_snapshot_callback();
                    },
                    CANCEL: function() {
                        $(this).dialog("close");
                    }
                }
            });
        });
    },

    create_snapshot_callback: function() {
        var modal = $("#create-snap-modal");
        var snap_name = $(".snapname", modal).val();
        var snapobj_name = $(".snapobjname", modal).val();
        var snap_desc = $(".snapdesc", modal).val();
        var consistent = $(".consistent", modal).is(":checked");
        var consolidate = $('.consolidate', modal).is(":checked");
        var data = { snap_name: snap_name,
            snapobj_name: snapobj_name,
            snap_desc: snap_desc,
            consistent: consistent,
            consolidate: consolidate
        };
        var ajaxOptions = {type: "POST", dataType: "json", data: data,
                           cache: false, url: sources.operations.create_snapshot};
        var done_callback = function() {
            $("#create-snap-modal").dialog("close");
            operations.updateTableCallback();
        }
        view.spinAjax(ajaxOptions, done_callback);
    },

    edit_snapshot: function(event) {
        var selElement = event.target;
        var actionsElement = $(selElement).closest('.actions');
        var serial = actionsElement.data("infinibox-serial");
        auth.spin_call_with_credentials(serial, function() {
            var elem = $("#edit-snap-modal");
            $(".snapname", elem).val(actionsElement.data('name'));
            $(".snapdesc", elem).val(actionsElement.data('desc'));
            elem.dialog({
                modal: true,
                width: 540,
                height: 450,
                resizable: false,
                buttons: {
                    DONE: function() {
                        var data = {snapshot_id: actionsElement.data('snapshot-id'),
                                    name: $(".snapname", elem).val(),
                                    desc: $(".snapdesc", elem).val()};
                        var ajaxOptions = {type: "POST", dataType: "json", data: data,
                                           cache: false, url: sources.operations.edit_snapshot};
                        var done_callback = function() {
                            $("#edit-snap-modal").dialog("close");
                            operations.updateTableCallback();
                        }
                        view.spinAjax(ajaxOptions, done_callback);
                    },
                    CANCEL: function() {
                        $(this).dialog("close");
                    }
                }
            });
        });
    },

    volume_info: function(event) {
        var popup_info = function(json) {
            var elem = $("#volume-info-modal");
            $("#volume-info-modal tr:nth-child(1) td:nth-child(2) label").text(json.return_value.snapshot_volume_name);
            $("#volume-info-modal tr:nth-child(2) td:nth-child(2) label").text(json.return_value.original_volume_name);
            $("#volume-info-modal tr:nth-child(3) td:nth-child(2) label").text(json.return_value.datastore_name);
            $("#volume-info-modal tr:nth-child(4) td:nth-child(2) label").text(json.return_value.snapshot_volume_size);
            $("#volume-info-modal tr:nth-child(5) td:nth-child(2) div").html(json.return_value.vm_list.join("<br>"));
            elem.dialog({
                modal: true,
                width: 600,
                height: 450,
                resizable: false,
                buttons: {
                    DONE: function() {
                        $(this).dialog("close");
                    }
                }
            });
        };
        var selElement = event.target;
        var actionsElement = $(selElement).closest('.actions');
        var serial = actionsElement.data("infinibox-serial");
        auth.spin_call_with_credentials(serial, function() {
            var data = {snapshot_id: actionsElement.data('snapshot-id')};
            var ajaxOptions = {type: "POST", dataType: "json", data: data, cache: false, url: sources.operations.volume_info};
            view.spinAjax(ajaxOptions, popup_info);
        });
    },

    setClickOnOperation: function(operationName, operationSources, parent) {
        var foreach = function(index, element) {
            var ajaxSource = operationSources[operationName]
            var selector = $(element);
            var callback_getter = operations[operationName + "_callback"]
            var callback = callback_getter(selector, ajaxSource);
            selector.click(callback);
        }
        $("." + operationName, parent).each(foreach);
    },

    init: function(operationSources) {
        init_actions_menu();
        operations.init_action_listeners();
        $("#snap-tree").fancytree();
    },

    init_action_listeners: function() {
        $(".create_snapshot_popup").on('click', operations.create_snapshot_popup);
        $(".snap-modal .modal-button, .snap-modal .snap-target-textbox").on('click', operations.open_target_treeview);
        $('body')
            .on('click', '.edit_snapshot', operations.edit_snapshot)
            .on('click', '.revert_snapshot', operations.revert_snapshot)
            .on('click', '.expose_snapshot', operations.revert_snapshot)
            .on('click', '.confirm_remove_snapshot', operations.confirm_remove_snapshot)
            .on('click', '.volume_info', operations.volume_info);
    }

}