#!/bin/sh
TARGETDIR=${1:-.}
HOSTNAME=`uname -n`
OPENSSL_CONF_DIR=/opt/vmware/etc/ssl
OPENSSL_CONF=$OPENSSL_CONF_DIR/openssl.conf
TMP_DIR=/var/tmp/tmp-certs

trap "rm -rf $TMP_DIR; exit 0" 0 1 2 15

echo "Generating SSL certificates for lighttpd in $TARGETDIR"
logger -- "Generating SSL certificates for lighttpd in $TARGETDIR"


if [ ! -f $OPENSSL_CONF ]
then
    echo "ERROR: Unable to find openssl configuration file, using defaults"
    mkdir -p $OPENSSL_CONF_DIR
    cat > $OPENSSL_CONF <<EOF
[req]
distinguished_name=user_dn
prompt=no

[user_dn]
CN=WARNING: Config file not found
EOF
fi

#openssl tools uses path in RANDFILE to write out a seeding file, this fixes 
#the "Unable to to write 'random state' message during key gen
RANDFILE=/root
export RANDFILE

mkdir -p $TMP_DIR;
chown root:root $TMP_DIR;
chmod 700 $TMP_DIR;

umask 0277; openssl req -x509 -days 10000 -newkey rsa:2048 \
   -nodes -config $OPENSSL_CONF   \
   -keyout $TMP_DIR/server.pem -out $TMP_DIR/server.pem > /dev/null 2>&1

cp $TMP_DIR/server.pem $TARGETDIR/server.pem
chmod 400 $TARGETDIR/server.pem
